@@ -9,6 +9,7 @@
 #ifndef __MORGOTHDBMA_CONOUT_H__
 #define __MORGOTHDBMA_CONOUT_H__
 #include <stdlib.h>
+#include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -18,7 +19,9 @@
 #define PATH 2048
 int system_printf(char*, ...);
 int debug(char*, ...);
+int sdebug(char*, ...);
 int fatal(char*, ...);
+int fatal_here(char*, int, char*, ...);
 int error(char*, ...);
 int warn(char*, ...);
 FILE* fopen_printf(char* mode, char* fmt, ...);
