@@ -621,8 +621,10 @@ int setup_combo(PGconn* c, GtkWidget* w, GtkWidget** cmb)
  /*free_p3c(&ptr,a,b);*/
  *cmb = gtk_combo_new();
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(*cmb)->entry),"Select DB from list");
+ gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(*cmb)->entry), FALSE);
  gtk_combo_set_popdown_strings(GTK_COMBO(*cmb), list);
  g_list_free(list);
+ //free_p3c(&ptr, row, col);	//XXX can we free it? WE CANT
  return 0;
 }
 
