@@ -24,7 +24,7 @@ gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
 }
 
 
-void test_gtk(PGconn* con)
+void test_gtk(PGconn* con, char* pass)
 {
     GtkWidget *window;
     GtkWidget *button;
@@ -52,7 +52,13 @@ void test_gtk(PGconn* con)
     free_p3c(&lout,a,b);
     printf("GET BUTTON DATA OK.\n");
     disconnect_db(&con);
-    system("dropdb pgxtest");
+    if (pass) con = connect_db("template1", pass);
+    else      con = connect_db("template1", NULL);
+    if (!con) { error("connect to template1\n"); return; }
+    usleep(50000);
+    res=execute_query(con, "DROP DATABASE pgxtest");
+    if (!res) { error("DROPPING DB: pgxtest"); return; }
+    disconnect_db(&con);
     gtk_init (NULL,NULL);
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title (GTK_WINDOW (window), "Hello Buttons!");
