@@ -12,5 +12,5 @@
 #include "query.h"
 void callback(GtkWidget*, gpointer);
 gint delete_event( GtkWidget*, GdkEvent*, gpointer);
-void test_gtk(PGconn* con);
+void test_gtk(PGconn* con, char*);
 #endif
