@@ -267,6 +267,7 @@ void subtree_DB_views()
  char ***ptr;
  int row,col,i;
  char errstr[MAX_QUERY_LENGTH];
+ char* tmps;
  GtkWidget* subtmp;
 #ifdef SMALLOC
  struct dmem_list* mem;
@@ -276,32 +277,37 @@ void subtree_DB_views()
 #ifdef SMALLOC
  if (!want_all_elems)
  dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
-		 "SELECT viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' ORDER BY viewname");
+		 "SELECT schemaname,viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' AND schemaname != 'information_schema' ORDER BY schemaname");
  else
  dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
-		 "SELECT viewname from pg_views ORDER BY viewname");
+		 "SELECT schemaname,viewname from pg_views ORDER BY schemaname");
 #else
  if (!want_all_elems)
  execute_printf(connection, &ptr, &row, &col, errstr, 0,
-		 "SELECT viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' ORDER BY viewname");
+		 "SELECT schemaname,viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' AND schemaname != 'information_schema' ORDER BY schemaname");
  else
  execute_printf(connection, &ptr, &row, &col, errstr, 0,
-		 "SELECT viewname from pg_views ORDER BY viewname");
+		 "SELECT schemaname,viewname from pg_views ORDER BY schemaname");
 #endif
  if (!ptr) return;
  if (row<0 || col<0)
    {
-    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\nThere: %s %d", errstr, HERE);
     return;
    }
  for (i=0;i<row;i++)
   {
-   subtmp = gtk_tree_item_new_with_label(ptr[i][0]);
+   tmps = malloc(strlen(ptr[i][0])+strlen(ptr[i][1])+3);
+   strcpy(tmps,ptr[i][0]);
+   strcat(tmps,".");
+   strcat(tmps,ptr[i][1]);
+   subtmp = gtk_tree_item_new_with_label(tmps);
    gtk_signal_connect_object(GTK_OBJECT(subtmp), "event",      GTK_SIGNAL_FUNC(dispatch_cmenu), GTK_OBJECT(components.cmenu_view));
    gtk_signal_connect(GTK_OBJECT(subtmp), "select",     GTK_SIGNAL_FUNC(select_final), mkptr("vs"));
    gtk_signal_connect(GTK_OBJECT(subtmp), "deselect",   GTK_SIGNAL_FUNC(select_final), mkptr("vd"));
    gtk_tree_append(GTK_TREE(components.subtree_views), subtmp);
    gtk_widget_show(subtmp);
+   free(tmps);
   }
  components.nviews=row;
 #ifdef SMALLOC
@@ -428,6 +434,82 @@ void refresh_tab_view(GtkWidget* w, gpointer data)
  browse_table(components.selected);
 }
 
+int numeric(char* val)
+{
+ int i;
+ float v;
+ if (!val) return 0;
+ i = sscanf(val,"%f", &v);
+ if (i==1) return 1;
+ else return 0;
+}
+
+void delete_current_row(GtkWidget* w, gpointer data)
+{
+ char*** ptr;
+ int row,col;
+ char errstr[MAX_QUERY_LENGTH+100];
+ char* querystr;
+ char* tmps;
+ char colname[STDSTRING];
+ int i,j,x;
+ int wbytes;
+ int nmrc;
+ j = 0;
+ i = components.last_row;
+ if (i<0) 
+ {
+	 gtk_dialog_printf("No row is selected!");
+	 return;
+ }
+ execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 1", components.selected);
+ if (row<0 || col<0)
+   {
+    fatal_here(HERE, "get column name failed");
+    return;
+   }
+ strcpy(colname, ptr[0][j]);
+ debug("delete row from field field: %d, %d %s:%s\n", i, j, colname,components.data[i+components.rrow][j+components.rcol]);
+ wbytes = STDSTRING;
+ for (x=0;x<col;x++) wbytes += (strlen(ptr[0][x])+1);
+ querystr = (char*)malloc(wbytes+1);
+ if (!querystr) fatal_here(HERE, "malloc failed");
+ sprintf(querystr, "DELETE FROM %s WHERE ", components.selected);
+ for (x=0;x<col;x++)
+ {
+  if (numeric(ptr[1][x])) nmrc = 1;
+  else nmrc = 0;
+  tmps = (char*)malloc(strlen(ptr[0][x])+strlen(components.data[i+components.rrow][x+components.rcol])+10);
+  if (!tmps) fatal_here(HERE, "malloc failed");
+  if (x < col-1)
+  {
+  	if (nmrc) sprintf(tmps, "%s = %s AND ", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  	else sprintf(tmps, "%s = '%s' AND ", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  }
+  else
+  {
+  	if (nmrc) sprintf(tmps, "%s = %s", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  	else sprintf(tmps, "%s = '%s'", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  }
+  strcat(querystr, tmps);
+  free(tmps);
+ }
+ debug("query: %d, %d, %s\n", components.rrow, components.rcol, querystr);
+ free_p3c(&ptr, row, col);
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, querystr);
+ if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Delete error!\n%s\n", errstr);
+    free(querystr);
+    return;
+   }
+ free_p3c(&ptr, row, col);
+ free(querystr);
+ destruct_tab_view(w,data);
+ browse_table(components.selected);
+ return ;
+}
+
 void destruct_tab_view(GtkWidget* w, gpointer data)
 {
  int i,rows,cols,j;
@@ -442,6 +524,8 @@ void destruct_tab_view(GtkWidget* w, gpointer data)
  components.button_refresh = NULL;
  if (components.button_update) destroy_widget(components.button_update);
  components.button_update = NULL;
+ if (components.button_delete) destroy_widget(components.button_delete);
+ components.button_delete = NULL;
  rows = components.rows;
  cols = components.cols;
  components.last_selected = NULL;
@@ -477,57 +561,105 @@ void destruct_tab_view(GtkWidget* w, gpointer data)
  destroy_pbar();
 }
 
-
-void drop_table(char* tabname)
+void drop_view(char* tabname, int force)
 {
  char*** ptr;
  int row,col;
+ int ans;
+ char errstr[MAX_QUERY_LENGTH+100];
+ debug("%s:%d drop_view",__FILE__,__LINE__);
+ execute_printf(connection, &ptr, &row, &col, errstr, 1, "DROP VIEW %s", tabname);
+ if (row<0 || col<0)
+   {
+    if (strstr(errstr,"CASCADE"))
+    {
+	if (force) ans = 1;
+	else ans = gtk_dialog_yes_no_printf("This view is referenced by other views\nDelete cascade (dangerous)?");
+	if (!ans) return;
+ 	execute_printf(connection, &ptr, &row, &col, errstr, 1, "DROP VIEW %s CASCADE", tabname);
+	if (row<0 || col<0)
+	{
+    		if (!force) gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+		else printf("Error dropping %s:\n%s\n", tabname,errstr);
+    		return;
+	}
+    }
+    else 
+    {
+    	if (!force) gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+	else printf("Error dropping %s:\n%s\n", tabname,errstr);
+    	return;
+    }
+   }
+ /*gtk_tree_remove_item(components.tree_item_tables, components.last_selected);*/
+ refresh_all();						/* FIXME hardcore!!! */
+ free_p3c(&ptr, row, col);
+ if (!force) gtk_dialog_printf("View %s dropped", tabname);
+}
+
+
+void drop_table(char* tabname, int force)
+{
+ char*** ptr;
+ int row,col;
+ int ans;
  char errstr[MAX_QUERY_LENGTH+100];
  debug("%s:%d drop_table",__FILE__,__LINE__);
  execute_printf(connection, &ptr, &row, &col, errstr, 1, "DROP TABLE %s", tabname);
  if (row<0 || col<0)
    {
-    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
-    return;
+    if (strstr(errstr,"CASCADE"))
+    {
+	if (force) ans = 1;
+	else ans = gtk_dialog_yes_no_printf("This table is referenced by other tables\nDelete cascade (dangerous)?");
+	if (!ans) return;
+ 	execute_printf(connection, &ptr, &row, &col, errstr, 1, "DROP TABLE %s CASCADE", tabname);
+	if (row<0 || col<0)
+	{
+    		if (!force) gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+		else printf("Error dropping %s:\n%s\n", tabname,errstr);
+    		return;
+	}
+    }
+    else 
+    {
+    	if (!force) gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+	else printf("Error dropping %s:\n%s\n", tabname,errstr);
+    	return;
+    }
    }
  /*gtk_tree_remove_item(components.tree_item_tables, components.last_selected);*/
  refresh_all();						/* FIXME hardcore!!! */
- gtk_dialog_printf("Table %s dropped", tabname);
+ free_p3c(&ptr, row, col);
+ if (!force) gtk_dialog_printf("Table %s dropped", tabname);
 }
 
-int numeric(char* val)
-{
- int i;
- float v;
- if (!val) return 0;
- i = sscanf(val,"%f", &v);
- if (i==1) return 1;
- else return 0;
-}
 
 void update_database(GtkWidget* w, gpointer data)
 {
  int i,j;
- char*** ptr;
- int row,col;
- char errstr[MAX_QUERY_LENGTH+100];
  char* gent;
- execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 1", components.selected);
- if (row<0 || col<0)
-   {
-    fatal_here(HERE, "get column name failed");
-    return;
-   }
+ int errs,err,succ;
+ errs = 0;
+ err  = 0;
+ succ = 0;
  for (i=1;i<components.rows;i++) 
  for (j=0;j<components.cols;j++)
    {
     gent = gtk_entry_get_text(GTK_ENTRY(components.table_entry[i][j]));
-    if (strcmp(gent,components.data[i][j])) changed_field(components.table_entry[i][j],(void*)((i<<0x10)+j));
+    if(strcmp(gent,components.data[i+components.rrow][j+components.rcol])) 
+     {
+	    err = changed_field(components.table_entry[i][j],(void*)((i<<0x10)+j+(1<<0x1F)));
+	    if (err==0) succ++;
+	    errs += err;
+     }
    }
- free_p3c(&ptr, row, col);
+ if (errs)
+ {
+	gtk_dialog_printf("There were %d update errors\n%d updates success\n", errs, succ);
+ }
 }
-
-void changed_field(GtkWidget* field, gpointer which)
+/*int changed_field(GtkWidget* field, gpointer which)
 {
  char*** ptr;
  int row,col;
@@ -537,20 +669,22 @@ void changed_field(GtkWidget* field, gpointer which)
  char* gent;
  int i,j;
  int nmrc;
- j = (int)which & 0xFFFF;
- i = ((int)which & 0xFFFF0000)>>0x10;
+ int out;
+ out = 0;
+ if ((int)which & (1<<0x1C)) out = 1;
+ j = (int)which & 0x7FFF;
+ i = ((int)which & 0x7FFF0000)>>0x10;
  execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 1", components.selected);
  if (row<0 || col<0)
    {
     fatal_here(HERE, "get column name failed");
-    return;
+    return 1;
    }
  strcpy(colname, ptr[0][j]);
  free_p3c(&ptr, row, col);
  printf("changed field: %d, %d %s:%s\n", i, j, colname,components.data[i][j]);
  gent = gtk_entry_get_text(GTK_ENTRY(field));
  if (!gent) fatal_here(HERE, "text is null"); 
- /*if (!strcmp(gent,"")) */
  if (numeric(components.data[i][j])) nmrc = 1;
  else nmrc = 0;
  if (!nmrc) sprintf(querystr, "UPDATE %s SET %s='%s' WHERE %s='%s'", components.selected, colname, gent, colname, components.data[i][j]);
@@ -559,9 +693,10 @@ void changed_field(GtkWidget* field, gpointer which)
  execute_printf(connection, &ptr, &row, &col, errstr, 0, querystr);
  if (row<0 || col<0)
    {
-    printf("DBMS warning: %s\n", errstr);
+    if (out) printf("DBMS warning: %s\n", errstr);
+    else gtk_dialog_printf_big("Update error!\n%s\n", errstr);
     gtk_entry_set_text(GTK_ENTRY(field), components.data[i][j]);
-    return;
+    return 1;
    }
  free_p3c(&ptr, row, col);
  free(components.data[i][j]);
@@ -573,6 +708,93 @@ void changed_field(GtkWidget* field, gpointer which)
 #endif
  if (!components.data[i][j]) fatal_here(HERE,"malloc failed");
  strcpy(components.data[i][j], gent);
+ return 0;
+}*/
+
+int changed_field(GtkWidget* field, gpointer which)
+{
+ char*** ptr;
+ int row,col;
+ char errstr[MAX_QUERY_LENGTH+100];
+ char* querystr;
+ char* tmps;
+ char colname[STDSTRING];
+ char* gent;
+ int i,j,x;
+ int nmrc;
+ int out;
+ int wbytes;
+ out = 0;
+ if ((int)which & (1<<0x1F)) out = 1;
+ j = (int)which & 0x7FFF;
+ i = ((int)which & 0x7FFF0000)>>0x10;
+ execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 2", components.selected);
+ if (row<0 || col<0)
+   {
+    fatal_here(HERE, "get column name failed");
+    return 1;
+   }
+ strcpy(colname, ptr[0][j]);
+ debug("changed field: %d, %d %s:%s\n", i, j, colname,components.data[i+components.rrow][j+components.rcol]);
+ gent = gtk_entry_get_text(GTK_ENTRY(field));
+ if (!gent) fatal_here(HERE, "text is null"); 
+ /*if (!strcmp(gent,"")) */
+ if (numeric(components.data[i+components.rrow][j+components.rcol])) nmrc = 1;
+ else nmrc = 0;
+ wbytes = STDSTRING;
+ for (x=0;x<col;x++) wbytes += (strlen(ptr[0][x])+1);
+ querystr = (char*)malloc(wbytes+1);
+ if (!querystr) fatal_here(HERE, "malloc failed");
+ if (!nmrc) sprintf(querystr, "UPDATE %s SET %s='%s' WHERE ", components.selected, colname, gent); //colname, components.data[i][j]);
+ else sprintf(querystr, "UPDATE %s SET %s=%s WHERE ", components.selected, colname, gent); //colname, components.data[i][j]);
+ for (x=0;x<col;x++)
+ {
+  debug("x = %d\n", x);
+  if (numeric(ptr[1][x])) nmrc = 1;
+  else nmrc = 0;
+  tmps = (char*)malloc(strlen(ptr[0][x])+strlen(components.data[i+components.rrow][x+components.rcol])+10);
+  if (!tmps) fatal_here(HERE, "malloc failed");
+  if (x < col-1)
+  {
+  	if (nmrc) sprintf(tmps, "%s = %s AND ", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  	else sprintf(tmps, "%s = '%s' AND ", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  }
+  else
+  {
+  	if (nmrc) sprintf(tmps, "%s = %s", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  	else sprintf(tmps, "%s = '%s'", ptr[0][x], components.data[i+components.rrow][x+components.rcol]);
+  }
+  strcat(querystr, tmps);
+  free(tmps);
+ }
+ debug("query: %d, %d, %s\n", components.rrow, components.rcol, querystr);
+ free_p3c(&ptr, row, col);
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, querystr);
+ if (row<0 || col<0)
+   {
+    if (out) printf("DBMS warning: %s\n", errstr);
+    else gtk_dialog_printf_big("Update error!\n%s\n", errstr);
+    gtk_entry_set_text(GTK_ENTRY(field), components.data[i+components.rrow][j+components.rcol]);
+    free(querystr);
+    return 1;
+   }
+ free_p3c(&ptr, row, col);
+ free(querystr);
+ free(components.data[i+components.rrow][j+components.rcol]);
+ components.data[i+components.rrow][j+components.rcol] = 0;
+#ifdef SMALLOC
+ components.data[i+components.rrow][j+components.rcol] = (char*)dmmalloc(strlen(gent)+1, HERE);
+#else
+ components.data[i+components.rrow][j+components.rcol] = (char*)mmalloc(strlen(gent)+1);
+#endif
+ if (!components.data[i+components.rrow][j+components.rcol]) fatal_here(HERE,"malloc failed");
+ strcpy(components.data[i+components.rrow][j+components.rcol], gent);
+ return 0;
+}
+
+void selected_field(GtkWidget* w, GdkEventAny* ev, gpointer which)
+{
+ components.last_row = (int)which;
 }
 
 void browse_table(char* tabname)
@@ -600,6 +822,7 @@ void browse_table(char* tabname)
  rrow=rcol=0;
  components.acols = col;
  components.arows = row;
+ components.last_row = -1;
  if (col>32 || row>512)
    {
     gtk_dialog_printf("Result is too big, rows=%d, columns=%d\nwill be truncated!", row, col);
@@ -633,7 +856,11 @@ void browse_table(char* tabname)
     if (i) gtk_entry_set_text(GTK_ENTRY(components.table_entry[i][j]), ptr[i+rrow][j+rcol]);
     else   gtk_entry_set_text(GTK_ENTRY(components.table_entry[i][j]), ptr[0][j+rcol]);
     if (!i) gtk_entry_set_editable(GTK_ENTRY(components.table_entry[i][j]), FALSE);
-    else gtk_signal_connect(GTK_OBJECT(components.table_entry[i][j]),"activate", GTK_SIGNAL_FUNC(changed_field), (void*)((i<<0x10)+j));
+    else 
+    {
+       	gtk_signal_connect(GTK_OBJECT(components.table_entry[i][j]),"activate", GTK_SIGNAL_FUNC(changed_field), (void*)((i<<0x10)+j));
+   	gtk_signal_connect(GTK_OBJECT(components.table_entry[i][j]),"focus_in_event", GTK_SIGNAL_FUNC(selected_field), (void*)(i));
+    }
     gtk_table_attach(GTK_TABLE (components.table_view), components.table_entry[i][j], j, j+1, i, i+1,GTK_FILL,GTK_FILL, 0, 0);
     gtk_widget_show(components.table_entry[i][j]);
     if (!cont) { gtk_dialog_printf("Interrupted!"); goto out_loop; }
@@ -652,24 +879,29 @@ void browse_table(char* tabname)
  
  components.button_destroy = gtk_button_new_with_label("Destroy");
  gtk_signal_connect(GTK_OBJECT(components.button_destroy),"clicked", GTK_SIGNAL_FUNC(destruct_tab_view), NULL);
- gtk_table_attach (GTK_TABLE(components.table), components.button_destroy, 20, 35, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE(components.table), components.button_destroy, 20, 32, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_destroy);
  
  components.button_save = gtk_button_new_with_label("Save");
  gtk_signal_connect(GTK_OBJECT(components.button_save),"clicked", GTK_SIGNAL_FUNC(save_results_mainwnd), NULL);
- gtk_table_attach (GTK_TABLE (components.table), components.button_save, 35, 50, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_save, 32, 44, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_save);
  
  components.button_refresh = gtk_button_new_with_label("Refresh");
  gtk_signal_connect(GTK_OBJECT(components.button_refresh),"clicked", GTK_SIGNAL_FUNC(refresh_tab_view), NULL);
- gtk_table_attach (GTK_TABLE (components.table), components.button_refresh, 50, 65, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_refresh, 44, 56, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_refresh);
  
  components.button_update = gtk_button_new_with_label("Update");
  gtk_signal_connect(GTK_OBJECT(components.button_update),"clicked", GTK_SIGNAL_FUNC(update_database), NULL);
- gtk_table_attach (GTK_TABLE (components.table), components.button_update, 65, 80, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_update, 56, 68, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_update);
  
+ components.button_delete = gtk_button_new_with_label("Delete Row");
+ gtk_signal_connect(GTK_OBJECT(components.button_delete),"clicked", GTK_SIGNAL_FUNC(delete_current_row), NULL);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_delete, 68, 80, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_widget_show(components.button_delete);
+ 
  set_wname_tab(tabname);
 }
 
@@ -684,9 +916,181 @@ void cmenu_browse(GtkWidget* w, gpointer data)
   destruct_tab_view(NULL,NULL);
   browse_table(components.selected);
  }
+ else if (components.seltype=='v')
+ {
+  destruct_tab_view(NULL,NULL);
+  browse_table(components.selected);
+ }
  else gtk_dialog_printf("Not yet implemented!");
 }
 
+void drop_views()
+{
+ char ***ptr;
+ int row,col,i;
+ char errstr[MAX_QUERY_LENGTH];
+ char* tmps;
+#ifdef SMALLOC
+ struct dmem_list* mem;
+ mem = NULL;
+#endif
+ debug("%s:%d subtree_DB_views",__FILE__,__LINE__);
+#ifdef SMALLOC
+ if (!want_all_elems)
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
+		 "SELECT schemaname,viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' AND schemaname != 'information_schema' ORDER BY schemaname");
+ else
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
+		 "SELECT schemaname,viewname from pg_views ORDER BY schemaname");
+#else
+ if (!want_all_elems)
+ execute_printf(connection, &ptr, &row, &col, errstr, 0,
+		 "SELECT schemaname,viewname from pg_views WHERE viewname NOT LIKE 'pg_%%' AND schemaname != 'information_schema' ORDER BY schemaname");
+ else
+ execute_printf(connection, &ptr, &row, &col, errstr, 0,
+		 "SELECT schemaname,viewname from pg_views ORDER BY schemaname");
+#endif
+ if (!ptr) return;
+ if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\nThere: %s %d", errstr, HERE);
+    return;
+   }
+ for (i=0;i<row;i++)
+  {
+   tmps = malloc(strlen(ptr[i][0])+strlen(ptr[i][1])+3);
+   strcpy(tmps,ptr[i][0]);
+   strcat(tmps,".");
+   strcat(tmps,ptr[i][1]);
+   drop_view(tmps, 1);
+   free(tmps);
+  }
+#ifdef SMALLOC
+ dmem_free_p3c(&mem, &ptr, row, col, HERE);
+ dshred_special_memlist(&mem, HERE);
+#else
+ free_p3c(&ptr, row, col);
+#endif
+}
+
+void drop_tables()
+{
+ char ***ptr;
+ int row,col,i;
+ char errstr[MAX_QUERY_LENGTH];
+#ifdef SMALLOC
+ struct dmem_list* mem;
+ mem = NULL;
+#endif
+ debug("%s:%d drop_tables",__FILE__,__LINE__);
+ if (!want_all_elems)
+#ifdef SMALLOC
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
+		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%'");
+ else
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,
+		 "SELECT tablename from pg_tables");
+#else
+ execute_printf(connection, &ptr, &row, &col, errstr, 0,
+		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%'");
+ else
+ execute_printf(connection, &ptr, &row, &col, errstr, 0,
+		 "SELECT tablename from pg_tables");
+#endif
+ if (!ptr) return;
+ if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    return;
+   }
+ for (i=0;i<row;i++) drop_table(ptr[i][0], 1);
+#ifdef SMALLOC
+ dmem_free_p3c(&mem, &ptr, row, col, HERE);
+ dshred_special_memlist(&mem, HERE);
+#else
+ free_p3c(&ptr, row, col);
+#endif
+}
+
+void cmenu_delete(GtkWidget* w, gpointer data)
+{
+  char* query;
+  char ***ptr;
+  int row,col;
+  char errstr[MAX_QUERY_LENGTH];
+#ifdef SMALLOC
+  struct dmem_list* mem;
+  mem = NULL;
+#endif
+  debug("%s:%d cmenu_delete",__FILE__,__LINE__);
+  query = delete_wizard(components.selected);
+  if (!query) 
+  {
+	gtk_dialog_printf("Delete wizard returned NULL query!");
+	return ;
+  }
+  debug("QUERY = %s\n", query);
+#ifdef SMALLOC
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,query);
+#else
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, query);
+#endif
+  if (row < 0 || col < 0)
+  {
+    gtk_dialog_printf_big("Delete error:\nReturn from DBMS: %s\n", errstr);
+    free(query);
+    return;
+  }
+  destruct_tab_view(NULL,NULL);
+/*  browse_table(components.selected);*/
+  free(query);
+#ifdef SMALLOC
+ dmem_free_p3c(&mem, &ptr, row, col, HERE);
+ dshred_special_memlist(&mem, HERE);
+#else
+ free_p3c(&ptr, row, col);
+#endif
+}
+
+void cmenu_insert(GtkWidget* w, gpointer data)
+{
+  char* query;
+  char ***ptr;
+  int row,col;
+  char errstr[MAX_QUERY_LENGTH];
+#ifdef SMALLOC
+  struct dmem_list* mem;
+  mem = NULL;
+#endif
+  debug("%s:%d cmenu_insert",__FILE__,__LINE__);
+  query = insert_wizard(components.selected);
+  if (!query) 
+  {
+	gtk_dialog_printf("Insert wizard returned NULL query!");
+	return ;
+  }
+  debug("QUERY = %s\n", query);
+#ifdef SMALLOC
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 0, HERE,query);
+#else
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, query);
+#endif
+  if (row < 0 || col < 0)
+  {
+    gtk_dialog_printf_big("Insert error:\nReturn from DBMS: %s\n", errstr);
+    free(query);
+    return;
+  }
+  destruct_tab_view(NULL,NULL);
+/*  browse_table(components.selected);*/
+  free(query);
+#ifdef SMALLOC
+ dmem_free_p3c(&mem, &ptr, row, col, HERE);
+ dshred_special_memlist(&mem, HERE);
+#else
+ free_p3c(&ptr, row, col);
+#endif
+}
 
 void cmenu_drop(GtkWidget* w, gpointer data)
 {
@@ -694,9 +1098,24 @@ void cmenu_drop(GtkWidget* w, gpointer data)
  if (components.seltype=='t')
  {
   destruct_tab_view(NULL,NULL);
-  drop_table(components.selected);
+  drop_table(components.selected, 0);
  }
- else gtk_dialog_printf("Not yet implemented!");
+ else if (components.seltype=='v')
+ {
+  destruct_tab_view(NULL,NULL);
+  drop_view(components.selected, 0);
+ }
+ else if (components.seltype=='T')
+ {
+  destruct_tab_view(NULL,NULL);
+  drop_tables();
+ }
+ else if (components.seltype=='V')
+ {
+  destruct_tab_view(NULL,NULL);
+  drop_views();
+ }
+ else gtk_dialog_printf("Not implemented/Not known!");
 }
 
 
@@ -706,60 +1125,68 @@ void create_context_menus()
  debug("%s:%d create_context_menus",__FILE__,__LINE__);
  /* TABLE */
  components.cmenu_table = gtk_menu_new();
- menu_item = gtk_menu_item_new_with_label("Browse table");
+ menu_item = gtk_menu_item_new_with_label("Browse table");		//DONE (but no dlete row)
  gtk_menu_append(GTK_MENU(components.cmenu_table), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Drop this table");
+ menu_item = gtk_menu_item_new_with_label("Drop this table");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_table), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_drop), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Browse Entire DB");
+ menu_item = gtk_menu_item_new_with_label("Browse Entire DB");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_table), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse_all), NULL);
  gtk_widget_show(menu_item);
+ menu_item = gtk_menu_item_new_with_label("Insert");			//DONE
+ gtk_menu_append(GTK_MENU(components.cmenu_table), menu_item);
+ gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_insert), NULL);
+ gtk_widget_show(menu_item);
+ menu_item = gtk_menu_item_new_with_label("Delete");			//DONE
+ gtk_menu_append(GTK_MENU(components.cmenu_table), menu_item);
+ gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_delete), NULL);
+ gtk_widget_show(menu_item);
  /* TABLE ENDS */
  /* TABLES */
  components.cmenu_tables = gtk_menu_new();
- menu_item = gtk_menu_item_new_with_label("Browse entire DB");
+ menu_item = gtk_menu_item_new_with_label("Browse entire DB");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_tables), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse_all), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Drop all tables");
+ menu_item = gtk_menu_item_new_with_label("Drop all tables");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_tables), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_drop), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("For tables");
+ menu_item = gtk_menu_item_new_with_label("Create table");
  gtk_menu_append(GTK_MENU(components.cmenu_tables), menu_item);
- gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_response), NULL);
+ gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(create_table), NULL);
  gtk_widget_show(menu_item);
  /* TABLE ENDS */
  /* VIEW */
  components.cmenu_view = gtk_menu_new();
- menu_item = gtk_menu_item_new_with_label("Browse View");
+ menu_item = gtk_menu_item_new_with_label("Browse View");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_view), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Drop this view");
+ menu_item = gtk_menu_item_new_with_label("Drop this view");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_view), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_drop), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Browse Entire DB");
+ menu_item = gtk_menu_item_new_with_label("Browse Entire DB");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_view), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse_all), mkptr("bla"));
  gtk_widget_show(menu_item);
  /* VIEW ENDS */
  /* VIEWS */
  components.cmenu_views = gtk_menu_new();
- menu_item = gtk_menu_item_new_with_label("Testing Context menu");
+ menu_item = gtk_menu_item_new_with_label("Testing Context menu");	//CREATE TABLE
  gtk_menu_append(GTK_MENU(components.cmenu_views), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_response), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Drop all views");
+ menu_item = gtk_menu_item_new_with_label("Drop all views");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_views), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_drop), NULL);
  gtk_widget_show(menu_item);
- menu_item = gtk_menu_item_new_with_label("Browse Entire DB");
+ menu_item = gtk_menu_item_new_with_label("Browse Entire DB");		//DONE
  gtk_menu_append(GTK_MENU(components.cmenu_views), menu_item);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cmenu_browse_all), mkptr("bla"));
  gtk_widget_show(menu_item);
@@ -1701,6 +2128,48 @@ int create_query_from_fname(char* fname, GtkWidget* txt)
  return 1;
 }
 
+void tdrop_table(GtkWidget* w, gpointer opt)
+{
+  debug("%s:%d create_table",__FILE__,__LINE__);
+  char* tab;
+  tab = select_table_wizard();
+  if (!tab) 
+  {
+	gtk_dialog_printf("Select table wizard returned NULL query!");
+	return ;
+  }
+ if (!strcmp(tab,"*all*")) 
+ {
+	 if (gtk_dialog_yes_no("Are you sure want to drop all tables?")) drop_tables();
+ }
+ else drop_table(tab, 0);
+ free(tab); 
+}
+
+void tdrop_view(GtkWidget* w, gpointer opt)
+{
+  debug("%s:%d create_table",__FILE__,__LINE__);
+  char* tab;
+  tab = select_view_wizard();
+  if (!tab) 
+  {
+	gtk_dialog_printf("Select view wizard returned NULL query!");
+	return ;
+  }
+ if (!strcmp(tab,"*all*")) 
+ {
+	 if (gtk_dialog_yes_no("Are you sure want to drop all views?")) drop_views();
+ }
+ else drop_view(tab, 0);
+ free(tab); 
+}
+
+void create_table(GtkWidget* w, gpointer opt)
+{
+ debug("%s:%d create_table",__FILE__,__LINE__);
+ fatal_here(HERE, "dummy: stub create_table NOT IMPLEMENTED FIXME\nWill be supported in final v0.30");	//FIXME
+}
+
 
 void execute_fast_one(GtkWidget* w, gpointer opt)
 {
@@ -2349,18 +2818,11 @@ void todo()
  printf("very carefully check dmem, struct dmem_list - ALL FUNCTIONS\n");
  printf("dmem rather checked, but m... *NOT*\n");
  printf("when DB has no tables, we cannot get to tables menu, is it OK?\n");
- printf("or add option in root-tree \"Tables\"?\n");
- printf("dodac wykrywanie mandarynki...\n");
- printf("drop all tables, when cannot drop table ask for: drop_cascade\n");
- printf("check errstr for 'depends on other relations' and ask user for confirm\n");
- printf("ALL NORAM & MINE FREE MUST BE FOLLOWED BE ZEROYING PTR, CHECK ALL MALLOCS\n");
  printf("localize all normall malloc and normal free and THINK about them...\n");
  printf("LIMIT 3; P233, P235, FUNCTIONS<-TRIGGERS,OPERATORS IN C, SQL\n");
- printf("na ktorej tablei trigger - p270, 277,w C p288, READ CONTRIB\n");
+ printf("na ktorej tabeli trigger - p270, 277,w C p288, READ CONTRIB\n");
  printf("read pgsql_book!\n");
  printf("pg_proc - proname, IN TREE (NO!-too much), BUT CAN BE CHOOSEN IN TRIG_WIZ\n");
- printf("DB_COPY, CREATE_FROM_FILE ETC when password used.\n");
- printf("VISUAL_EDIT>> ADD ROWS, DROP ROWS\n");
 }
 
 
@@ -2412,6 +2874,7 @@ int main(int lb, char **par)
      else sprintf(winname, "PgXexplorer: NULL");
      gtk_window_set_title (GTK_WINDOW(window), winname);
     }
+  system("killall -9 pbar");
   gtk_main();
   if (pass) free(pass);
   pass = NULL;
