@@ -34,9 +34,10 @@ struct main_components
  GtkWidget* button_destroy;
  GtkWidget* button_refresh;
  GtkWidget* button_update;
+ GtkWidget* button_delete;
  GtkWidget*** table_entry;
  GtkWidget* last_selected;
- int ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol;
+ int ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol,last_row;
  /* cols HOW MUCH ON WINDOW, acols ALL COLUMNS, RCOL, ROTATE_COL, OFFSET COL ON WINDOW */
  char selected[STDSTRING];
  char seltype;
