@@ -16,6 +16,7 @@ int ping_conn;
 extern int timeout;
 
 
+
 int connAlive(PGconn* c)
 {
  PGresult* r;
@@ -49,22 +50,24 @@ void disconnect_db(PGconn** c)
 }
 
 
-PGconn* connect_db(char* arg)
+PGconn* connect_db(char* arg, char* pass)
 {
  PGconn* conn;
  char* strarg;
  conn=0;
  debug("%s:%d connect_db", __FILE__,__LINE__);
- strarg = (char*)malloc(strlen(arg)+32);
+ strarg = (char*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
  debug("\nconnect to database %s\n", arg);
- sprintf(strarg,"dbname=template1");
+ if (!pass) sprintf(strarg,"dbname=template1");
+ else	     sprintf(strarg,"dbname=template1 password='%s'", pass);
  alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
  if (!conn) { error("cannot connect to template1 DB"); free(strarg); return NULL; }
  if (PQstatus(conn)==CONNECTION_BAD) {  error("cannot connect to template1 DB"); free(strarg); return NULL; }
  disconnect_db(&conn);
- sprintf(strarg,"dbname=%s", arg);
+ if (!pass) sprintf(strarg,"dbname=%s", arg);
+ else sprintf(strarg,"dbname=%s password='%s'", arg,pass);
  alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
@@ -81,21 +84,23 @@ PGconn* connect_db(char* arg)
 }
 
 
-PGconn* gtk_connect_db(char* arg, char* errstr)
+PGconn* gtk_connect_db(char* arg, char* pass, char* errstr)
 {
  PGconn* conn;
  char* strarg;
  conn=0;
  debug("%s:%d gtk_connect_db", __FILE__,__LINE__);
- strarg = (char*)malloc(strlen(arg)+16);
- sprintf(strarg,"dbname=template1");
+ strarg = (char*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
+ if (!pass) sprintf(strarg,"dbname=template1");
+ else       sprintf(strarg,"dbname=template1 password='%s'", pass);
  alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
  if (!conn) { sprintf(errstr,"cannot connect to template1 DB"); free(strarg); return NULL; }
  if (PQstatus(conn)==CONNECTION_BAD) {  sprintf(errstr,"cannot connect to template1 DB"); free(strarg); return NULL; }
  disconnect_db(&conn);
- sprintf(strarg,"dbname=%s", arg);
+ if (!pass) sprintf(strarg,"dbname=%s",arg);
+ else       sprintf(strarg,"dbname=%s password='%s'", arg, pass);
  alarm_clock(timeout);
  conn = PQconnectdb(strarg);
  alarm_stop();
