@@ -343,6 +343,83 @@ static const char * xpm_quit[] =
 "   crrrrrrrrc   ",
 "    cccccccc    ",
 };
+
+static const char * xpm_ctable[] =
+{
+"16 16 4 1",
+"       c None",
+"c      c #000000000000",
+"r      c #FFFF00000000",
+"k      c #FFFFFFFFFFFF",
+"                ",
+"  ccccccccccccc ",
+"  c  c  c  c  c ",
+"  c  c  c  c  c ",
+"  ccccccccccccc ",
+"  c  c  c  c  c ",
+"  c  c  c  c  c ",
+"  ccccccccccccc ",
+"  c  c  c  c  c ",
+"  c  c  c  c  c ",
+"  ccccccccccccc ",
+"  c  c  c  c  c ",
+"  c  c  c  c  c ",
+"  ccccccccccccc ",
+"                ",
+"                ",
+};
+
+static const char * xpm_dtable[] =
+{
+"16 16 4 1",
+"       c None",
+"c      c #000000000000",
+"r      c #FFFF00000000",
+"k      c #FFFFFFFFFFFF",
+"rr              ",
+" rrcccccccccccc ",
+"  rr c  c  c  c ",
+"  crrc  c  c  c ",
+"  ccrrccccccccc ",
+"  c  rr c  c  c ",
+"  c  crrc  c  c ",
+"  cccccrrcccccc ",
+"  c  c  rr c  c ",
+"  c  c  crrc  c ",
+"  ccccccccrrccc ",
+"  c  c  c  rr c ",
+"  c  c  c  crrc ",
+"  cccccccccccrr ",
+"              r ",
+"                ",
+};
+
+static const char * xpm_dview[] =
+{
+"16 16 4 1",
+"       c None",
+"c      c #000000000000",
+"r      c #00000000FFFF",
+"k      c #FFFFFFFFFFFF",
+"rr              ",
+" rrcccccccccccc ",
+"  rr c  c  c  c ",
+"  crrc  c  c  c ",
+"  ccrrccccccccc ",
+"  c  rr c  c  c ",
+"  c  crrc  c  c ",
+"  cccccrrcccccc ",
+"  c  c  rr c  c ",
+"  c  c  crrc  c ",
+"  ccccccccrrccc ",
+"  c  c  c  rr c ",
+"  c  c  c  crrc ",
+"  cccccccccccrr ",
+"              r ",
+"                ",
+};
+
+
 void get_toolbar(GtkWidget* w, GtkWidget **tb);
 void about_window_new();
 
@@ -679,7 +756,7 @@ void about_window_new()
  gtk_widget_grab_focus(button);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(about_clback), NULL);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(about_clback), NULL);
- gtk_window_set_title(GTK_WINDOW(window), "PgXexplorer version 0.14 alpha");
+ gtk_window_set_title(GTK_WINDOW(window), "PgXexplorer version 0.27 alpha");
  gtk_widget_show(window);
  gtk_main();
  gtk_widget_destroy(window);
@@ -747,6 +824,19 @@ void get_toolbar(GtkWidget* w, GtkWidget **tb)
  pix = gtk_pixmap_new(pix_new, mask);
  gtk_toolbar_append_item(GTK_TOOLBAR(*tb), NULL, "Fast Execute Command", "Private",pix,execute_fast,(gpointer)(3));
  gtk_toolbar_append_space(GTK_TOOLBAR(*tb));
+ pix_new = gdk_pixmap_colormap_create_from_xpm_d
+	 (w->window,gcm,&mask,&style->bg[GTK_STATE_NORMAL],(gchar **)xpm_ctable);
+ pix = gtk_pixmap_new(pix_new, mask);
+ gtk_toolbar_append_item(GTK_TOOLBAR(*tb), NULL, "Create table wizard", "Private",pix,create_table,NULL);
+ pix_new = gdk_pixmap_colormap_create_from_xpm_d
+	 (w->window,gcm,&mask,&style->bg[GTK_STATE_NORMAL],(gchar **)xpm_dtable);
+ pix = gtk_pixmap_new(pix_new, mask);
+ gtk_toolbar_append_item(GTK_TOOLBAR(*tb), NULL, "Drop table(s)", "Private",pix,tdrop_table,NULL);
+ pix_new = gdk_pixmap_colormap_create_from_xpm_d
+	 (w->window,gcm,&mask,&style->bg[GTK_STATE_NORMAL],(gchar **)xpm_dview);
+ pix = gtk_pixmap_new(pix_new, mask);
+ gtk_toolbar_append_item(GTK_TOOLBAR(*tb), NULL, "Drop view(s)", "Private",pix,tdrop_view,NULL); 
+ gtk_toolbar_append_space(GTK_TOOLBAR(*tb));
  pix_new = gdk_pixmap_colormap_create_from_xpm_d
 	 (w->window,gcm,&mask,&style->bg[GTK_STATE_NORMAL],(gchar **)xpm_about);
  pix = gtk_pixmap_new(pix_new, mask);
